import React, { Suspense, useEffect, useRef, useState } from 'react'
import { useTranslation } from 'react-i18next'

import { Box, Button, CircularProgress } from '@mui/material'
import Page from 'components/page'

import { useHistory } from 'react-router-dom'
import {
  CountryCode,
  getCountries,
  getCountryCallingCode,
} from 'libphonenumber-js'
import format from 'date-fns/format'
import { splitPhoneNumber, submitJsonOpts, fetchJsonOpts } from 'utils'
import OverviewGuestButton from '../../components/overviewGuestButton'
import { GuestRegisterCallableMethods } from './guestRegisterCallableMethods'
import { GuestRegisterData, GuestConsentData } from './enteredGuestData'
import { GuestInviteInformation } from './guestDataForm'
import AuthenticationMethod from './authenticationMethod'
import GuestRegisterStep from './steps/register'
import GuestConsentStep from './steps/consent'
import GuestSuccessStep from './steps/success'

enum SubmitState {
  NotSubmitted,
  Submitted,
  SubmittedError,
}

enum Step {
  RegisterStep,
  ConsentStep,
  SuccessStep,
}

type InvitationData = {
  person: {
    first_name: string
    last_name: string
    email?: string
    mobile_phone?: string
    fnr?: string
    passport?: string
    feide_id?: string
  }
  sponsor: {
    first_name: string
    last_name: string
  }
  role: {
    ou_name_nb: string
    ou_name_en: string
    role_name_nb: string
    role_name_en: string
    start: string
    end: string
    comments: string
  }
  meta: {
    session_type: string
  }
}

/*
 * When the guest reaches this page he has already an invite ID set in his session.
 */
export default function GuestRegister() {
  const { t } = useTranslation(['common'])
  const history = useHistory()
  // TODO On submit successful the user should be directed to some page telling
  // eslint-disable-next-line @typescript-eslint/no-unused-vars
  const [submitState, setSubmitState] = useState<SubmitState>(
    SubmitState.NotSubmitted
  )
  const guestRegisterRef = useRef<GuestRegisterCallableMethods>(null)
  // TODO Set step when user moves between pages
  const [activeStep, setActiveStep] = useState(0)

  const [initialGuestData, setInitialGuestData] =
    useState<GuestInviteInformation>({
      first_name: '',
      last_name: '',
      ou_name_en: '',
      ou_name_nb: '',
      role_name_en: '',
      role_name_nb: '',
      role_start: '',
      role_end: '',
      comment: '',
      email: '',
      mobile_phone: '',
      dateOfBirth: undefined,
      fnr: '',
      passport: '',
      passportNationality: '',
      countryForCallingCode: '',
      authentication_method: AuthenticationMethod.Invite,
    })

  const [guestRegisterData, setGuestRegisterData] =
    useState<GuestRegisterData | null>(null)
  const [guestConsentData, setGuestConsentData] =
    useState<GuestConsentData | null>(null)

  const fetchInvitationData = async () => {
    const response = await fetch('/api/ui/v1/invited/', fetchJsonOpts())
    if (!response.ok) {
      return
    }
    const data: InvitationData = await response.json()

    const authenticationMethod =
      data.meta.session_type === 'invite'
        ? AuthenticationMethod.Invite
        : AuthenticationMethod.Feide

    const [countryCode, nationalNumber] = data.person.mobile_phone
      ? splitPhoneNumber(data.person.mobile_phone)
      : ['', '']

    let extractedCountryCode = ''
    if (countryCode) {
      const matchingCountries = getCountries().find(
        (value) => getCountryCallingCode(value) === countryCode
      )

      if (matchingCountries && matchingCountries.length > 0) {
        extractedCountryCode = matchingCountries.toString()
      }
    }

    let passportNumber = ''
    let passportNationality = ''
    if (data.person.passport) {
      const parts = data.person.passport.split('-', 1)
      if (parts.length === 2) {
        ;[passportNationality, passportNumber] = parts
      }
    }

    setInitialGuestData({
      first_name: data.person.first_name,
      last_name: data.person.last_name,
      ou_name_en: data.role.ou_name_en,
      ou_name_nb: data.role.ou_name_nb,
      role_name_en: data.role.role_name_en,
      role_name_nb: data.role.role_name_nb,
      role_start: data.role.start,
      role_end: data.role.end,
      comment: data.role.comments,

      email: data.person.email,
      feide_id: data.person.feide_id,
      fnr: data.person.fnr,

      passport: passportNumber,
      passportNationality,

      mobile_phone_country_code: countryCode,
      mobile_phone: nationalNumber,
      countryForCallingCode: extractedCountryCode,

      authentication_method: authenticationMethod,
    })
  }

  useEffect(() => {
    fetchInvitationData()
  }, [])

  const handleNext = () => {
    if (activeStep === Step.RegisterStep) {
      if (guestRegisterRef.current) {
        guestRegisterRef.current.doSubmit()
      }
    }
  }

  const handleBack = () => {
    if (activeStep === Step.ConsentStep) {
      setActiveStep(Step.RegisterStep)
    }
  }

  const makePayload = (
    registerData: GuestRegisterData,
    consentData: GuestConsentData
  ): any => {
    const payload: any = {
      person: {},
    }
    payload.person.mobile_phone = `+${getCountryCallingCode(
      registerData.mobilePhoneCountry as CountryCode
    )}${registerData.mobilePhone}`

    if (
      initialGuestData.authentication_method === AuthenticationMethod.Invite
    ) {
      // The authentication method is Invite, so the name does not come from a
      // trusted third-party source, and the user can update it
      payload.person.first_name = registerData.firstName
      payload.person.last_name = registerData.lastName
    }

    if (registerData.passportNumber && registerData.passportNationality) {
      payload.person.passport = `${registerData.passportNationality}-${registerData.passportNumber}`
    }

    if (registerData.nationalIdNumber) {
      payload.person.fnr = registerData.nationalIdNumber
    }

    if (consentData.consents) {
      payload.person.consents = consentData.consents
    }

    if (registerData.dateOfBirth) {
      payload.person.date_of_birth = format(
        registerData.dateOfBirth as Date,
        'yyyy-MM-dd'
      )
    }

    return payload
  }

  const submitPayload = () => {
    if (!guestRegisterData) {
      setActiveStep(Step.RegisterStep)
      return
    }
    if (!guestConsentData) {
      setActiveStep(Step.ConsentStep)
      return
    }
    const payload: any = makePayload(guestRegisterData, guestConsentData)
    console.log('submitting payload', payload)
    fetch('/api/ui/v1/invited/', submitJsonOpts('POST', payload))
      .then((response) => {
        if (response.ok) {
          setSubmitState(SubmitState.Submitted)
          setActiveStep(Step.SuccessStep)
        } else {
          setSubmitState(SubmitState.SubmittedError)
          console.error(`Server responded with status: ${response.status}`)
        }
      })
      .catch((error) => {
        setSubmitState(SubmitState.SubmittedError)
        console.error(error)
      })
  }

  const handleForwardFromRegister = (registerData: GuestRegisterData): void => {
    console.log('handleForwardFromRegister')
    setGuestRegisterData(registerData)
    setActiveStep(Step.ConsentStep)
  }

  const handleForwardFromConsent = (consentData: GuestConsentData): void => {
    console.log('handleForwardFromConsent')
    setGuestConsentData(consentData)
    if (!guestRegisterData) {
      setActiveStep(Step.RegisterStep)
      return
    }

    if (!guestConsentData) {
      setActiveStep(Step.ConsentStep)
      return
    }
    submitPayload()
  }

  const handleCancel = () => {
    history.push('/')
  }

  return (
    <Page>
      <OverviewGuestButton />
      {/* Step: Registration  */}
      {activeStep === Step.RegisterStep && (
        <Box sx={{ width: '100%' }}>
          <GuestRegisterStep
            nextHandler={handleForwardFromRegister}
            initialGuestData={initialGuestData}
            registerData={guestRegisterData}
            ref={guestRegisterRef}
          />
        </Box>
      )}

      {/* Step: Consent */}
      {activeStep === Step.ConsentStep && (
        <Box sx={{ width: '100%' }}>
          <Suspense fallback={<CircularProgress />}>
            <GuestConsentStep
              nextHandler={handleForwardFromConsent}
              ref={guestRegisterRef}
            />
          </Suspense>
        </Box>
      )}

      {/* Step: Success */}
      {activeStep === Step.SuccessStep && <GuestSuccessStep />}

      {/* Navigation */}
      <Box
        sx={{
          display: 'flex',
          flexDirection: 'row',
          pt: 2,
          color: 'primary.main',
          paddingBottom: '1rem',
        }}
      >
        {activeStep === Step.RegisterStep && (
          <Button
            data-testid="button-next"
            color="secondary"
            sx={{ mr: 1 }}
            onClick={handleNext}
          >
            {t('button.next')}
          </Button>
        )}
        {activeStep === Step.ConsentStep && (
          <Button
            data-testid="button-black"
            sx={{ color: 'theme.palette.secondary', mr: 1 }}
            onClick={handleBack}
          >
            {t('button.back')}
          </Button>
        )}

        {activeStep !== Step.SuccessStep && (
          <Button color="secondary" onClick={handleCancel}>
            {t('button.cancel')}
          </Button>
        )}
      </Box>
    </Page>
  )
}
